require 'cequel'

module Dock
  class Cequel < Base
    def create(attributes = {})
      model.create(attributes)
    end
    def find(key_values)
      find!(key_values)
    rescue Cequel::Record::RecordNotFound
      nil
    end
    def find!(key_values)
      model.find(*key_values)
    end
    def all(options = {})
        construct_scope(options).to_a
    end
    def first(options = {})
      construct_scope(options).first
    end
    def id
      'id'
    end
    def update(search_key, find_by, update_key, by_value)
      entry = model.find(update_key).posts.find(find_by)
      entry.update_attributes!(update_key by_value)
    end
    def destroy(record)
      record.destroy
    end
    def associations()

    end
    def model_name
      model.class.name
    end
    def column_names
      model.columns.map { |column| column.name }
    end
    # Checked
    def count(options = {})
      all(options).count
    end
    def encoding
      'UTF-8'
    end
    def belongs_to()

    end
    def has_many()

    end
    def scoped?
      false
    end
    def embedded?
      false
    end
    def cyclic?
      false
    end
    def supports_joins?
      false
    end
    def properties()

    end
    private
      def construct_scope(options)
        conditions, _, limit, _ =
          extract_conditions!(options.deep_symbolize_keys)
        scope = model.all
        scope = apply_secondary_index_scope(scope, conditions) ||
          apply_primary_key_scope(scope, conditions)
        scope = scope.limit(limit) if limit
        scope
      end

      def extract_conditions!(options)
        super.tap do |_, order, _, offset|
          if order.present?
            fail ArgumentError,
                 "Cassandra does not support ordering of results by " \
                 "arbitrary columns"
          end
          if offset.present?
            fail ArgumentError, 'Cassandra does not support row offsets'
          end
        end
      end

      def apply_primary_key_scope(scope, conditions)
        conditions = conditions.dup
        conditions.assert_valid_keys(*model.key_column_names)

        model.key_column_names.each do |column_name|
          column_value = conditions.delete(column_name)
          break unless column_value
          scope = scope[column_value]
        end

        assert_conditions_empty!(conditions)

        scope
      end

      def apply_secondary_index_scope(scope, conditions)
        return unless conditions.one?
        condition_column = model.reflect_on_column(conditions.keys.first)
        if condition_column.data_column? && condition_column.indexed?
          scope.where(*conditions.first)
        else
        end
      end

      def assert_conditions_empty!(conditions)
        unless conditions.empty?
          fail ArgumentError,
               "Invalid columns for conditions: #{conditions.keys.join(', ')}"
      end
    end
  end
end
