require 'mongo_mapper'

module Dock
  class MongoMapper < Base
    def create(attributes = {})
        model.create!(attributes)
    end
    def find(id)
      model.find!(wrap_key(id))
    end
    def find!(id)
      model.first({ :id => wrap_key(id) })
    end
    def all(conditions = {})
      conditions, order, limit, offset = extract_conditions!(conditions)
      conditions = conditions.merge(:sort => order) unless order.nil?
      conditions = conditions.merge(:limit => limit) unless limit.nil?
      conditions = conditions.merge(:offset => offset) unless limit.nil? || offset.nil?
      model.all(conditions_to_fields(conditions))
    end
    def first(conditions = {})
      conditions, order = extract_conditions!(conditions)
      conditions = conditions.merge(:sort => order) unless order.nil?
      model.first(conditions_to_fields(conditions))
    end
    def id
      '_id'
    end
    def update(search_key, find_by, update_key, by_value)
      entry = model.where(search_key find_by)
      entry.set(update_key => by_value)
      entry.reload
    end
    def destroy(object)
        object.destroy if valid_object?(object)
    end
    def associations()

    end
    def model_name
      model.class.name
    end
    def column_names
      model.fields.keys
    end
    # Checked
    def count(conditions = {})
      all(options).count
    end
    def encoding
      'UTF-8'
    end
    def belongs_to()

    end
    def has_many()

    end
    # Checked
    def scoped?
      model.scoped
    end
    # Checked
    def embedded?
      model.relations.values.detect { |a| a.macro.to_sym == :embedded_in }
    end
    # Checked
    def cyclic?
      model.cyclic?
    end
    # Checked
    def supports_joins
      false
    end
    def properties()

    end
    protected

      # converts and documents to ids
      def conditions_to_fields(conditions)
        conditions.inject({}) do |fields, (key, value)|
          if value.is_a?(MongoMapper::Document) && model.key?("#{key}_id")
            fields.merge("#{key}_id" => value.id)
          else
            fields.merge(key => value)
          end
        end
      end
    end
  end
