require 'sequel'

module Dock
  class Sequel < Base
    def create()
      associated_objects = []
      attrs = {}

      model.db.transaction do
        attributes.each do |col, value|
          if model.associations.include?(col)
            Array(value).each{|v| associated_objects << [association_method(col), v]}
          else
            attrs.merge!(col => value) # pass it on
          end
        end

        obj = model.create(attrs) # create main obj
        associated_objects.each do |m,o|
          obj.send(m, o)
        end
        obj.save
      end # transaction
    end
    # Checked
    def find(id)
      model.find(wrap_key(model.primary_key => id))
    end
    # Checked
    def find!(id)
      model[wrap_key(id)] || raise(Error, "#{model.name} not found with #{model.primary_key} of #{wrap_key(id)}")
    end
    # Checked
    def all(options = {})
      conditions, order = extract_conditions!(options)
      model.filter(conditions_to_hash(conditions)).order(*order_clause(order)).all
    end
    # Checked
    def first(options = {})
      conditions, order = extract_conditions!(options)
      model.filter(conditions_to_hash(conditions)).order(*order_clause(order)).first
    end
    # Checked
    def id
      'primary_key'
    end
    def update(search_key, find_by, update_key, by_value)
      entry = model.where(search_key, find_by)
      entry.update(update_key by_value)
    end
    # Checked
    def destroy(object)
      object.destroy && true if valid_object?(object)
    end
    def associations()

    end
    # Checked
    def model_name
      model.class.name
    end
    # Checked
    def column_names
      model.columns
    end
    # Checked
    def count(options = {})
      all(options).count
    end
    def encoding
      'UTF-8'
    end
    def belongs_to()

    end
    def has_many()

    end
    def scoped?
      false
    end
    def embedded?
      false
    end
    def cyclic?
      true
    end
    # Checked
    def supports_joins?
      true
    end
    def properties()

    end
    protected

      def conditions_to_hash(conditions)
        conditions.inject({}) do |cond_hash, (col, value)|
          # make sure the key is a symbol
          col = col.to_sym

          if value.is_a?(Sequel::Model)
            # look up the column name for the assoc
            key = model.association_reflection(col)[:key]
            cond_hash.merge(key => value.id)
          else
            cond_hash.merge(col => value)
          end
        end
      end

      def order_clause(order)
        m = order.map {|pair| pair.first.send(pair.last)}
        m.empty? ? nil : m
      end

      def association_method(col)
        assoc = model.association_reflection(col)
        case assoc[:type]
        when :one_to_many, :many_to_many
          assoc.add_method
        else # when :many_to_one, :one_to_one
          assoc.setter_method
        end
      end
  end
end